Map = Map or {}
Map.__index = Map
function Map.new(construct, ...)
    local instance = setmetatable({}, Map)
    if construct and Map.constructor then Map.constructor(instance, ...) end
    return instance
end
function Map.constructor(self,other)
    self.items = {}
    self.size = 0
    if __TS__InstanceOf(other, Map) then
        self.size = other.size
        for _, kvp in ipairs(other:entries()) do
            do
                self.items[kvp[0+1]] = kvp[1+1]
            end
        end
    else
        if other~=nil then
            self.size = #other
            for _, kvp in ipairs(other) do
                do
                    self.items[kvp[0+1]] = kvp[1+1]
                end
            end
        end
    end
end
function Map.clear(self)
    self.items = {}
    self.size = 0
    return
end
function Map.delete(self,key)
    local contains = self:has(key)

    if contains then
        self.size = (self.size-1)
    end
    self.items[key] = nil
    return contains
end
function Map.entries(self)
    local out = {}

    for key, _ in pairs(self.items) do
        do
            out[#out+1] = {key,self.items[key]}
        end
    end
    return out
end
function Map.forEach(self,callback)
    for key, _ in pairs(self.items) do
        do
            callback(self.items[key],key,self)
        end
    end
    return
end
function Map.get(self,key)
    return self.items[key]
end
function Map.has(self,key)
    return self.items[key]~=nil
end
function Map.keys(self)
    local out = {}

    for key, _ in pairs(self.items) do
        do
            out[#out+1] = key
        end
    end
    return out
end
function Map.set(self,key,value)
    if (not self:has(key)) then
        self.size = (self.size+1)
    end
    self.items[key] = value
    return self
end
function Map.values(self)
    local out = {}

    for key, _ in pairs(self.items) do
        do
            out[#out+1] = self.items[key]
        end
    end
    return out
end
