Set = Set or {}
Set.__index = Set
function Set.new(construct, ...)
    local instance = setmetatable({}, Set)
    if construct and Set.constructor then Set.constructor(instance, ...) end
    return instance
end
function Set.constructor(self,other)
    self.items = {}
    self.size = 0
    if __TS__InstanceOf(other, Set) then
        self.size = other.size
        for _, value in ipairs(other:values()) do
            do
                self.items[value] = true
            end
        end
    else
        if other~=nil then
            self.size = #other
            for _, value in ipairs(other) do
                do
                    self.items[value] = true
                end
            end
        end
    end
end
function Set.add(self,value)
    if (not self:has(value)) then
        self.size = (self.size+1)
    end
    self.items[value] = true
    return self
end
function Set.clear(self)
    self.items = {}
    self.size = 0
    return
end
function Set.delete(self,value)
    local contains = self:has(value)

    if contains then
        self.size = (self.size-1)
    end
    self.items[value] = nil
    return contains
end
function Set.entries(self)
    local out = {}

    for key, _ in pairs(self.items) do
        do
            out[#out+1] = {key,key}
        end
    end
    return out
end
function Set.forEach(self,callback)
    for key, _ in pairs(self.items) do
        do
            callback(key,key,self)
        end
    end
    return
end
function Set.has(self,value)
    return self.items[value]==true
end
function Set.keys(self)
    local out = {}

    for key, _ in pairs(self.items) do
        do
            out[#out+1] = key
        end
    end
    return out
end
function Set.values(self)
    local out = {}

    for key, _ in pairs(self.items) do
        do
            out[#out+1] = key
        end
    end
    return out
end
