"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Transpiler_51_1 = require("./Transpiler.51");
const ts = require("typescript");
class LuaTranspiler52 extends Transpiler_51_1.LuaTranspiler51 {
    /** @override */
    transpileLoopBody(node) {
        this.loopStack.push(this.genVarCounter);
        this.genVarCounter++;
        let result = this.indent + "do\n";
        this.pushIndent();
        result += this.transpileStatement(node.statement);
        this.popIndent();
        result += this.indent + "end\n";
        result += this.indent + `::__continue${this.loopStack.pop()}::\n`;
        return result;
    }
    /** @override */
    transpileContinue(node) {
        return this.indent + `goto __continue${this.loopStack[this.loopStack.length - 1]}\n`;
    }
    /** @override */
    transpileUnaryBitOperation(node, operand) {
        switch (node.operator) {
            case ts.SyntaxKind.TildeToken:
                return `bit32.bnot(${operand})`;
        }
    }
    /** @override */
    transpileBitOperation(node, lhs, rhs) {
        switch (node.operatorToken.kind) {
            case ts.SyntaxKind.AmpersandToken:
                return `bit32.band(${lhs},${rhs})`;
            case ts.SyntaxKind.BarToken:
                return `bit32.bor(${lhs},${rhs})`;
            case ts.SyntaxKind.CaretToken:
                return `bit32.bxor(${lhs},${rhs})`;
            case ts.SyntaxKind.LessThanLessThanToken:
                return `bit32.lshift(${lhs},${rhs})`;
            case ts.SyntaxKind.GreaterThanGreaterThanToken:
                return `bit32.rshift(${lhs},${rhs})`;
            case ts.SyntaxKind.GreaterThanGreaterThanGreaterThanToken:
                return `bit32.arshift(${lhs},${rhs})`;
        }
    }
    /** @override */
    transpileDestructingAssignmentValue(node) {
        return `table.unpack(${this.transpileExpression(node)})`;
    }
}
exports.LuaTranspiler52 = LuaTranspiler52;
//# sourceMappingURL=Transpiler.52.js.map